import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:natinfo_flutter/providers/operational_mode_provider.dart';
import 'package:natinfo_flutter/providers/theme_provider.dart';
import 'package:provider/provider.dart';
import 'package:natinfo_flutter/config/app_config.dart';
import 'repositories/natinf_repository.dart';
import 'pages/data_initialization_screen.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  final repository = NatinfRepository();
  await repository.init();
  runApp(MyRoot(repository: repository));
}

class NoAnimationPageTransitionsBuilder extends PageTransitionsBuilder {
  @override
  Widget buildTransitions<T>(
      PageRoute<T> route, BuildContext context,
      Animation<double> animation, Animation<double> secondaryAnimation,
      Widget child) {
    return child;
  }
}

class MyRoot extends StatelessWidget {
  final NatinfRepository repository;
  const MyRoot({super.key, required this.repository});

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        Provider<NatinfRepository>.value(value: repository),
        ChangeNotifierProvider(create: (_) => ThemeProvider()),
        ChangeNotifierProvider(create: (_) => OperationalModeProvider()),
      ],
      child: MyApp(repository: repository),
    );
  }
}

class MyApp extends StatelessWidget {
  final NatinfRepository repository;
  const MyApp({super.key, required this.repository});

  @override
  Widget build(BuildContext context) {
    return Consumer2<ThemeProvider, OperationalModeProvider>(
      builder: (context, themeProvider, opProvider, child) {
        if (opProvider.opMode) {
          SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
        } else {
          SystemChrome.setPreferredOrientations(DeviceOrientation.values);
        }
        final pageTransitionsTheme = opProvider.opMode
          ? PageTransitionsTheme(builders: {
              TargetPlatform.android: NoAnimationPageTransitionsBuilder(),
              TargetPlatform.iOS: NoAnimationPageTransitionsBuilder(),
            })
          : ThemeData.light().pageTransitionsTheme;
        return MaterialApp(
          title: 'NATINFo',
          theme: ThemeData.light().copyWith(
            primaryColor: themeProvider.primaryColor,
            appBarTheme: AppBarTheme(
              color: themeProvider.primaryColor,
              foregroundColor: themeProvider.primaryTextColor,
            ),
            pageTransitionsTheme: pageTransitionsTheme,
          ),
          darkTheme: ThemeData.dark().copyWith(
            primaryColor: themeProvider.primaryColor,
            appBarTheme: AppBarTheme(
              color: themeProvider.primaryColor,
              foregroundColor: themeProvider.primaryTextColor,
            ),
            pageTransitionsTheme: pageTransitionsTheme,
          ),
          themeMode: themeProvider.isDarkMode ? ThemeMode.dark : ThemeMode.light,
          home: DataInitializationScreen(repository: repository),
        );
      },
    );
  }
}